from django.db import models
from django.utils.translation import gettext_lazy as _

from colorfield.fields import ColorField

from aleksis.core.mixins import ExtensibleModel
from aleksis.core.models import Group, Person
from aleksis.core.util.model_helpers import ICONS


class GradeSet(ExtensibleModel):
    """Set of grades."""

    name = models.CharField(verbose_name=_("Name"), max_length=255, unique=True)
    groups = models.ManyToManyField(
        "core.Group",
        verbose_name=_("Groups using this grade set"),
        blank=True,
        null=True,
        related_name="grade_sets",
    )


class GradeChoice(ExtensibleModel):
    """Single grade choice in a set of grades."""

    grade_set = models.ForeignKey(
        GradeSet,
        on_delete=models.CASCADE,
        verbose_name=_("Grade set"),
        related_name="grade_choices",
    )
    name = models.CharField(verbose_name=_("Name"), max_length=255)
    value = models.CharField(verbose_name=_("Value"), max_length=255)
    order = models.PositiveSmallIntegerField(verbose_name=_("Order"))

    class Meta:
        verbose_name = _("Grade choice")
        verbose_name_plural = _("Grade choices")
        constraints = [
            models.UniqueConstraint(
                fields=["grade_set", "name"],
                name="unique_gradeset_name",
            ),
            models.UniqueConstraint(
                fields=["grade_set", "value"],
                name="unique_gradeset_value",
            ),
        ]


class EffortType(ExtensibleModel):
    """Abstract type that can be rated."""

    name = models.CharField(verbose_name=_("Name"), max_length=255)
    color = ColorField(verbose_name=_("Color"), blank=True)
    icon = models.CharField(
        max_length=50, blank=True, choices=[("", "")] + ICONS, verbose_name=_("Icon")
    )
    default = models.BooleanField(verbose_name=_("Default"))

    def save(self, *args, **kwargs):
        if self.default:
            # select all other active items
            qs = EffortType.objects.filter(default=True)
            # except self (if self already exists)
            if self.pk:
                qs = qs.exclude(pk=self.pk)
            # and deactivate them
            qs.update(default=False)

        super().save(*args, **kwargs)

    class Meta:
        verbose_name = _("Effort type")
        verbose_name_plural = _("Effort types")
        constraints = [
            models.UniqueConstraint(
                fields=["default"],
                condition=models.Q(default=True),
                name="only_one_default_effort_type",
            )
        ]


class Effort(ExtensibleModel):
    """Concrete occurrence of an effort type."""

    name = models.CharField(verbose_name=_("Name"), max_length=255)
    weight = models.PositiveSmallIntegerField(verbose_name=_("Weight"), default=1)
    effort_type = models.ForeignKey(
        EffortType,
        on_delete=models.CASCADE,
        verbose_name=_("Effort type"),
        related_name="efforts",
    )
    group = models.ForeignKey(
        Group,
        on_delete=models.PROTECT,
        verbose_name=_("Group"),
        related_name="efforts",
    )
    grade_set = models.ForeignKey(
        GradeSet,
        on_delete=models.CASCADE,
        verbose_name=_("Grade set"),
        related_name="+",
    )


class Grade(ExtensibleModel):
    """Grade of a person for an effort."""

    person = models.ForeignKey(
        Person,
        on_delete=models.CASCADE,
        verbose_name=_("Person"),
        related_name="grades",
    )
    grade = models.ForeignKey(
        GradeChoice,
        on_delete=models.CASCADE,
        verbose_name=_("Grade"),
        related_name="+",
    )
    effort = models.ForeignKey(
        Effort,
        on_delete=models.CASCADE,
        verbose_name=_("Effort"),
        related_name="grades",
    )
