from django.core.exceptions import PermissionDenied
from django.db.models import Q

from graphene_django.types import DjangoObjectType
from guardian.shortcuts import get_objects_for_user

from aleksis.core.models import Group
from aleksis.core.schema.base import (
    BaseBatchCreateMutation,
    BaseBatchDeleteMutation,
    BaseBatchPatchMutation,
    DjangoFilterMixin,
    PermissionsTypeMixin,
)
from aleksis.core.util.core_helpers import get_site_preferences, has_person

from ..models import Effort as EffortModel
from ..models import EffortType as EffortTypeModel
from .shared_secret import (
    SharedSecretBatchCreateMixin,
    SharedSecretBatchDeleteMixin,
    SharedSecretBatchPatchMixin,
    SharedSecretObjectType,
)


class EffortTypeType(
    PermissionsTypeMixin, DjangoFilterMixin, DjangoObjectType
):
    class Meta:
        model = EffortTypeModel
        fields = ("id", "name", "color", "icon", "default")
        filter_fields = {
            "id": ["exact"],
            "name": ["icontains", "exact"],
        }

    @classmethod
    def get_queryset(cls, queryset, info):
        if info.context.user.has_perm("maka.view_efforttypes_rule"):
            return queryset
        raise PermissionDenied()


class EffortTypeBatchCreateMutation(BaseBatchCreateMutation):
    class Meta:
        model = EffortTypeModel
        permissions = ("maka.add_efforttype_rule",)
        only_fields = (
            "name",
            "color",
            "icon",
            "default",
        )


class EffortTypeBatchDeleteMutation(BaseBatchDeleteMutation):
    class Meta:
        model = EffortTypeModel
        permissions = ("maka.delete_efforttype_rule",)


class EffortTypeBatchPatchMutation(BaseBatchPatchMutation):
    class Meta:
        model = EffortTypeModel
        permissions = ("maka.edit_efforttype_rule",)
        only_fields = (
            "id",
            "name",
            "color",
            "icon",
            "default",
        )


class EffortType(SharedSecretObjectType, PermissionsTypeMixin, DjangoFilterMixin, DjangoObjectType):
    class Meta:
        model = EffortModel
        fields = ("id", "name", "weight", "effort_type", "group", "grade_set", "grades")
        filter_fields = {
            "id": ["exact"],
            "name": ["icontains", "exact"],
            "group": ["exact", "in"],
        }

    @classmethod
    def get_queryset(cls, queryset, info):
        if info.context.user.has_perm("maka.view_effort"):
            return queryset
        elif has_person(info.context.user):
            owned_groups = info.context.user.person.owner_of.all()
            permission_groups = get_objects_for_user(info.context.user, "core.view_efforts_group", Group)

            groups = permission_groups.union(owned_groups).union(
                Group.objects.filter(parent_groups__in=owned_groups)
            )

            if get_site_preferences()["maka__view_own_efforts"]:
                groups = groups.union(info.context.user.person.member_of.all())

            return queryset.filter(group__in=groups.values_list("id", flat=True))
        
        raise PermissionDenied()


class EffortBatchCreateMutation(SharedSecretBatchCreateMixin, BaseBatchCreateMutation):
    class Meta:
        model = EffortModel
        permissions = ("maka.add_effort_rule",)
        only_fields = (
            "name",
            "effort_type",
            "group",
            "grade_set",
        )


class EffortBatchDeleteMutation(SharedSecretBatchDeleteMixin, BaseBatchDeleteMutation):
    class Meta:
        model = EffortModel
        permissions = ("maka.delete_effort_rule",)


class EffortBatchPatchMutation(SharedSecretBatchPatchMixin, BaseBatchPatchMutation):
    class Meta:
        model = EffortModel
        permissions = ("maka.edit_effort_rule",)
        only_fields = (
            "id",
            "name",
            "effort_type",
            "group",
            "grade_set",
        )
