from django.core.exceptions import PermissionDenied

from graphene_django.types import DjangoObjectType
from guardian.shortcuts import get_objects_for_user

from aleksis.core.models import Group
from aleksis.core.schema.base import (
    BaseBatchCreateMutation,
    BaseBatchDeleteMutation,
    BaseBatchPatchMutation,
    DjangoFilterMixin,
    PermissionsTypeMixin,
)
from aleksis.core.util.core_helpers import get_site_preferences, has_person

from ..models import Grade
from .shared_secret import (
    SharedSecretBatchCreateMixin,
    SharedSecretBatchDeleteMixin,
    SharedSecretBatchPatchMixin,
    SharedSecretObjectType,
)


class GradeType(SharedSecretObjectType, PermissionsTypeMixin, DjangoFilterMixin, DjangoObjectType):
    class Meta:
        model = Grade
        fields = ("id", "person", "grade", "effort")
        filter_fields = {
            "id": ["exact"],
            "effort": ["exact", "in"],
            "effort__group": ["exact", "in"],
            "name__lel": ["icontains", "exact"],
        }

    @classmethod
    def get_queryset(cls, queryset, info):
        if info.context.user.has_perm("maka.view_grade"):
            return queryset
        elif has_person(info.context.user):
            owned_groups = info.context.user.person.owner_of.all()
            permission_groups = get_objects_for_user(info.context.user, "core.view_efforts_group", Group)

            groups = permission_groups.union(owned_groups).union(
                Group.objects.filter(parent_groups__in=owned_groups)
            )

            if get_site_preferences()["maka__view_own_grades"]:
                groups = groups.union(info.context.user.person.member_of.all())

            return queryset.filter(effort__group__in=groups.values_list("id", flat=True))

        raise PermissionDenied()


class GradeBatchCreateMutation(SharedSecretBatchCreateMixin, BaseBatchCreateMutation):
    class Meta:
        model = Grade
        permissions = ("maka.add_grade_rule",)
        only_fields = (
            "person",
            "grade",
            "effort",
        )


class GradeBatchDeleteMutation(SharedSecretBatchDeleteMixin, BaseBatchDeleteMutation):
    class Meta:
        model = Grade
        permissions = ("maka.delete_grade_rule",)


class GradeBatchPatchMutation(SharedSecretBatchPatchMixin, BaseBatchPatchMutation):
    class Meta:
        model = Grade
        permissions = ("maka.edit_grade_rule",)
        only_fields = (
            "id",
            "person",
            "grade",
            "effort",
        )
