from django.contrib.auth.models import User
from django.db.models import Q

from rules import predicate

from aleksis.core.models import Group, Person
from aleksis.core.util.predicates import check_object_permission

from ..models import Effort, Grade


@predicate
def is_effort_group_member(user: User, obj: Effort):
    """Predicate for members of an effort.

    Checks whether the person linked to the user is a member in the group
    linked to the effort.
    """
    if obj:
        return user.person in list(obj.group.members.all())
    return False


@predicate
def is_effort_group_owner(user: User, obj: Effort):
    """Predicate for group owners of an effort.

    Checks whether the person linked to the user is a owner of the group
    (or its respective parent groups) linked to the effort.
    """
    if obj:
        return user.person in list(obj.group.owners.all()) or obj.group.parent_groups.filter(owners=user.person).exists()
    return False


def has_effort_group_perm(perm: str):
    """Predicate builder for permissions on efforts.

    Checks whether a user has a permission on the group of an effort.
    """
    name = f"has_effort_group_perm:{perm}"

    @predicate(name)
    def fn(user: User, obj: Effort) -> bool:
        return check_object_permission(user, perm, obj.group)

    return fn


@predicate
def is_grade_person(user: User, obj: Grade):
    """Predicate checking whether person of user is the person linked to the grade."""

    if obj and user.person == obj.person:
        return True
    return False


@predicate
def is_grade_group_owner(user: User, obj: Grade):
    """Predicate for group owners of the effort linked to a grade.
    
    Checks whether the person linked to the user is a owner of the group
    (or its respective parent groups) linked to the effort linked to
    the grade.
    """

    if obj:
        return is_effort_group_owner(user, obj.effort)
    return False


def has_grade_group_perm(perm: str):
    """Predicate builder for permissions on grades.

    Checks whether a user has a permission on the group of the effort
    linked to the grade.
    """
    name = f"has_grade_group_perm:{perm}"

    @predicate(name)
    def fn(user: User, obj: Grade) -> bool:
        return check_object_permission(user, perm, obj.effort.group)

    return fn


@predicate
def has_any_group_ownership(user: User):
    """Predicate checking whether the person linked to the user owns any groups."""

    return Group.objects.filter(owners=user.person).exists()
